import pandas as pd
from .regression_processor import RegressionProcessor
import numpy as np
import torch
from torch.utils.data import (DataLoader, TensorDataset)
import os
from pytorch_pretrained_bert import BertTokenizer, BertForSequenceClassification
from multiprocessing import Pool, cpu_count
from .convert_examples_to_features import convert_example_to_feature
from tqdm import tqdm



def preprocess(text, multiprocessing, Process_count, chunksize):
    text1 = pd.DataFrame(text, columns=['body'])
    train_df_bert = pd.DataFrame({
        'id': range(len(text)),
        'label': 0,
        'alpha': ['a'] * text1.shape[0],
        'text': text1['body'].replace(r'\n', ' ', regex=True)
    })
    train_df_bert.to_csv('train_regression.tsv', sep='\t', index=False, header=False)

    processor = RegressionProcessor()
    train_examples = processor.get_train_examples('')
    train_examples_len = len(train_examples)
    os.remove('train_regression.tsv')

    OUTPUT_MODE = 'regression'
    MAX_SEQ_LENGTH = 128

    tokenizer = BertTokenizer.from_pretrained('bert-base-cased', do_lower_case=False)

    label_map = {'0.0': 0}
    train_examples_for_processing = [(example, label_map, MAX_SEQ_LENGTH, tokenizer, OUTPUT_MODE) for example in
                                     train_examples]

    process_count = Process_count
    if multiprocessing == 'False':
        with Pool(process_count) as p:
            train_features = list(tqdm(
                p.imap(convert_example_to_feature, train_examples_for_processing),
                total=train_examples_len))
    elif multiprocessing == 'True':
        with Pool(process_count) as p:
            train_features = list(tqdm(
                p.imap(convert_example_to_feature, train_examples_for_processing, chunksize),
                total=train_examples_len))
    return train_features


import pandas as pd
import numpy as np
import torch
from torch.utils.data import (DataLoader, TensorDataset)
import os
from pytorch_pretrained_bert import BertTokenizer
from pytorch_transformers import BertForSequenceClassification
from .preprocess_data import preprocess
from multiprocessing import Pool, cpu_count
from .convert_examples_to_features import convert_example_to_feature
from tqdm import tqdm
from .regression_processor import RegressionProcessor
from .download_features import fetch_pretrained_model
import logging
from os.path import expanduser
logging.basicConfig(format='%(asctime)s - %(levelname)s - %(name)s -   %(message)s',
                    datefmt='%m/%d/%Y %H:%M:%S',
                    level=logging.INFO)

logger = logging.getLogger(__name__)


class Agreementr:
    def __init__(self,
                 device=torch.device("cuda" if torch.cuda.is_available() else "cpu"),
                 is_paralleled=False, BATCH_SIZE = 128, CPU_COUNT=1, CHUNKSIZE=1):
        self.device = device if isinstance(device, torch.device) else torch.device(device)
        model_path = os.path.join(os.path.dirname(__file__),"agreement_model.bin")
        self.model_type = 'agreementr'

        if not os.path.isfile(model_path):
            logger.info(f'Model {self.model_type} does not exist at {model_path}. Try to download it now.')
            model = 'agreement_model'
            fetch_pretrained_model(model, model_path)

        if self.device.type == "cpu":
            model_state_dict = torch.load(model_path, map_location=self.device.type)
        else:
            model_state_dict = torch.load(model_path)
        self.model = BertForSequenceClassification.from_pretrained('bert-base-cased', state_dict=model_state_dict, num_labels=1)
        if is_paralleled:
            if self.device.type == "cpu":
                print("Data parallel is not available with cpus")
            else:
                self.model = torch.nn.DataParallel(self.model)

        self.model.to(device)
        self.model.eval()
        self.batch_size = BATCH_SIZE
        self.cpu_count = CPU_COUNT
        self.chunksize = CHUNKSIZE

    def predict(self,text):
        text1 = pd.DataFrame(text, columns=['body'])
        train_df_bert = pd.DataFrame({
            'id': range(len(text)),
            'label': 0,
            'alpha': ['a'] * text1.shape[0],
            'text': text1['body'].replace(r'\n', ' ', regex=True)
        })
        train_df_bert.to_csv('train_regression.tsv', sep='\t', index=False, header=False)

        processor = RegressionProcessor()
        train_examples = processor.get_train_examples('')
        train_examples_len = len(train_examples)
        os.remove('train_regression.tsv')

        OUTPUT_MODE = 'regression'
        MAX_SEQ_LENGTH = 128

        tokenizer = BertTokenizer.from_pretrained('bert-base-cased', do_lower_case=False)

        label_map = {'0.0': 0}
        train_examples_for_processing = [(example, label_map, MAX_SEQ_LENGTH, tokenizer, OUTPUT_MODE) for example in
                                         train_examples]

        process_count = self.cpu_count
        chunksize = self.chunksize
        #global train_features


        with Pool(process_count) as p:
            train_features = list(tqdm(
                p.imap(convert_example_to_feature, train_examples_for_processing, chunksize),
                total=train_examples_len))

        all_input_ids = torch.tensor([f.input_ids for f in train_features], dtype=torch.long)
        all_input_mask = torch.tensor([f.input_mask for f in train_features], dtype=torch.long)
        all_segment_ids = torch.tensor([f.segment_ids for f in train_features], dtype=torch.long)
        all_label_ids = torch.tensor([f.label_id for f in train_features], dtype=torch.float)

        bert_data = TensorDataset(all_input_ids, all_input_mask, all_segment_ids, all_label_ids)
        bert_dataloader = DataLoader(bert_data, batch_size=self.batch_size)
        preds = []

        if self.device.type == 'cpu':
            for step, batch in enumerate(tqdm(bert_dataloader)):
                batch = tuple(t.to(self.device) for t in batch)
                input_ids, input_mask, segment_ids, label_ids = batch
                with torch.no_grad():
                    logits = self.model(input_ids, segment_ids, input_mask, labels=None)
                # logits = logits.detach().cpu()
                if len(preds) == 0:
                    preds.append(logits)
                else:
                    preds[0] = np.append(
                        preds[0], logits, axis=0)
            preds = [item for sublist in preds for item in sublist]
            preds = np.squeeze(np.array(preds[0]))
        else:
            for step, batch in enumerate(tqdm(bert_dataloader)):
                batch = tuple(t.cuda() for t in batch)
                input_ids, input_mask, segment_ids, label_ids = batch
                with torch.no_grad():
                    logits = self.model(input_ids, segment_ids, input_mask, labels=None)

                logits = logits[0]
                logits = logits.detach().cpu()
                if len(preds) == 0:
                    preds.append(logits)
                else:
                    preds[0] = np.append(
                        preds[0], logits, axis=0)
            preds = [item for sublist in preds for item in sublist]
            preds = np.squeeze(np.array(preds))
        return preds

























from __future__ import absolute_import, division, print_function

import csv
import os
import sys
import logging

logger = logging.getLogger()
csv.field_size_limit(2147483647) # Increase CSV reader's field limit incase we have long text.


class InputExample(object):
    """A single training/test example for simple sequence classification."""

    def __init__(self, guid, text_a, text_b=None, label=None):
        """Constructs a InputExample.
        Args:
            guid: Unique id for the example.
            text_a: string. The untokenized text of the first sequence. For single
            sequence tasks, only this sequence must be specified.
            text_b: (Optional) string. The untokenized text of the second sequence.
            Only must be specified for sequence pair tasks.
            label: (Optional) string. The label of the example. This should be
            specified for train and dev examples, but not for test examples.
        """
        self.guid = guid
        self.text_a = text_a
        self.text_b = text_b
        self.label = label


class DataProcessor(object):
    """Base class for data converters for sequence classification data sets."""

    def get_train_examples(self, data_dir):
        """Gets a collection of `InputExample`s for the train set."""
        raise NotImplementedError()

    def get_dev_examples(self, data_dir):
        """Gets a collection of `InputExample`s for the dev set."""
        raise NotImplementedError()

    def get_labels(self):
        """Gets the list of labels for this data set."""
        raise NotImplementedError()

    @classmethod
    def _read_tsv(cls, input_file, quotechar=None):
        """Reads a tab separated value file."""
        with open(input_file, "r", encoding="utf-8") as f:
            reader = csv.reader(f, delimiter="\t", quotechar=quotechar)
            lines = []
            for line in reader:
                if sys.version_info[0] == 2:
                    line = list(unicode(cell, 'utf-8') for cell in line)
                lines.append(line)
            return lines


class RegressionProcessor(DataProcessor):

    def get_train_examples(self, data_dir):
        """See base class."""
        return self._create_examples(
            self._read_tsv(os.path.join(data_dir, "train_regression.tsv")), "train")

    def get_labels(self):
        """See base class."""
        return ["1"]

    def _create_examples(self, lines, set_type):
        """Creates examples for the training and dev sets."""
        examples = []
        for (i, line) in enumerate(lines):
            guid = "%s-%s" % (set_type, i)
            text_a = line[3]
            label = line[1]
            examples.append(
                InputExample(guid=guid, text_a=text_a, text_b=None, label=label))
        return examples



from .agreementr import Agreementr

ar = None

def predict(text):
    global ar
    if ar is None:
        try:
            ar = Agreementr()
        except Exception as e:
            print("Failed to initialize Agreementr")
            print(e)

    return ar.predict(text)


import pandas as pd
import numpy as np
import torch
from torch.utils.data import (DataLoader, TensorDataset)
import os
from pytorch_pretrained_bert import BertTokenizer, BertForSequenceClassification
from multiprocessing import Pool, cpu_count
import convert_examples_to_features
from tqdm import tqdm
from regression_processor import RegressionProcessor
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")


def predict(text):

    text1 = pd.DataFrame(text, columns=['body'])
    train_df_bert = pd.DataFrame({
        'id': range(len(text)),
        'label': 0,
        'alpha': ['a'] * text1.shape[0],
        'text': text1['body'].replace(r'\n', ' ', regex=True)
    })
    train_df_bert.to_csv('train_regression.tsv', sep='\t', index=False, header=False)

    processor = RegressionProcessor()
    train_examples = processor.get_train_examples('')
    train_examples_len = len(train_examples)
    os.remove('train_regression.tsv')

    OUTPUT_MODE = 'regression'
    MAX_SEQ_LENGTH = 128

    tokenizer = BertTokenizer.from_pretrained('bert-base-cased', do_lower_case=False)

    label_map = {'0.0': 0}
    train_examples_for_processing = [(example, label_map, MAX_SEQ_LENGTH, tokenizer, OUTPUT_MODE) for example in
                                     train_examples]

    process_count = cpu_count() - 1
    if __name__ == '__main__':
        if train_examples_len > 10000:
            with Pool(process_count) as p:
                train_features = list(tqdm(
                    p.imap(convert_examples_to_features.convert_example_to_feature, train_examples_for_processing),
                    total=train_examples_len))
        else:
            with Pool(process_count) as p:
                train_features = list(tqdm(
                    p.imap(convert_examples_to_features.convert_example_to_feature, train_examples_for_processing),
                    total=train_examples_len))

    all_input_ids = torch.tensor([f.input_ids for f in train_features], dtype=torch.long)
    all_input_mask = torch.tensor([f.input_mask for f in train_features], dtype=torch.long)
    all_segment_ids = torch.tensor([f.segment_ids for f in train_features], dtype=torch.long)
    all_label_ids = torch.tensor([f.label_id for f in train_features], dtype=torch.float)

    # using 'cpu' to predict
    if device.type == 'cpu':
        model_state_dict = torch.load(
            'data/agreement_model.bin', map_location='cpu')
    # else: using 'gpu'
    else:
        model_state_dict = torch.load(
            'data/agreement_model.bin')
    model = BertForSequenceClassification.from_pretrained('bert-base-cased', state_dict=model_state_dict,
                                                          num_labels=1)
    # model = DataParallel(model)
    model.to(device)
    model.eval()

    preds = []
    bert_data = TensorDataset(all_input_ids, all_input_mask, all_segment_ids, all_label_ids)
    bert_dataloader = DataLoader(bert_data, batch_size=256)

    if device.type == 'cpu':
        for step, batch in enumerate(tqdm(bert_dataloader)):
            batch = tuple(t.to(device) for t in batch)
            input_ids, input_mask, segment_ids, label_ids = batch
            with torch.no_grad():
                logits = model(input_ids, segment_ids, input_mask, labels=None)
            # logits = logits.detach().cpu()
            if len(preds) == 0:
                preds.append(logits)
            else:
                preds[0] = np.append(
                    preds[0], logits, axis=0)
        preds = [item for sublist in preds for item in sublist]
        preds = np.squeeze(np.array(preds))
    else:
        for step, batch in enumerate(tqdm(bert_dataloader)):
            batch = tuple(t.cuda() for t in batch)
            input_ids, input_mask, segment_ids, label_ids = batch
            with torch.no_grad():
                logits = model(input_ids, segment_ids, input_mask, labels=None)

            logits = logits[0]
            logits = logits.detach().cpu()
            if len(preds) == 0:
                preds.append(logits)
            else:
                preds[0] = np.append(
                    preds[0], logits, axis=0)
        preds = [item for sublist in preds for item in sublist]
        preds = np.squeeze(np.array(preds))
    return preds


preds = predict(['how much keys for victor black Tachyon ?'])
print(preds)


import os
import logging
import tempfile
from tqdm import tqdm
import requests
import shutil
logging.basicConfig(format='%(asctime)s - %(levelname)s - %(name)s -   %(message)s',
                    datefmt='%m/%d/%Y %H:%M:%S',
                    level=logging.INFO)

logger = logging.getLogger(__name__)


def fetch_pretrained_model(model_name, model_path):
    # Edited from https://github.com/huggingface/pytorch-pretrained-BERT/blob/68a889ee43916380f26a3c995e1638af41d75066/pytorch_pretrained_bert/file_utils.py
    # TODO: check whether the license from huggingface works with ours
    PRETRAINED_MODEL_ARCHIVE_MAP = {
        'agreement_model': ['http://jurgens.people.si.umich.edu/models/agreement_model.bin']
    }
    model = 'agreement_model'
    assert model_name in PRETRAINED_MODEL_ARCHIVE_MAP
    model_urls = PRETRAINED_MODEL_ARCHIVE_MAP[model_name]

    download_flag = False
    for idx, model_url in enumerate(model_urls):
        try:
            temp_file = tempfile.NamedTemporaryFile()
            logger.info(f'{model_path} not found in cache, downloading from {model_url} to {temp_file.name}')

            req = requests.get(model_url, stream=True)
            content_length = req.headers.get('Content-Length')
            total = int(content_length) if content_length is not None else None
            progress = tqdm(unit="KB", total=round(total / 1024))
            for chunk in req.iter_content(chunk_size=1024):
                if chunk:  # filter out keep-alive new chunks
                    progress.update(1)
                    temp_file.write(chunk)
            progress.close()
            temp_file.flush()
            temp_file.seek(0)
            download_flag = True
        except Exception as e:
            logger.warning(f'Download from {idx + 1}/{len(model_urls)} mirror failed with an exception of\n{str(e)}')
            try:
                temp_file.close()
            except Exception as e_file:
                logger.warning(f'temp_file failed with an exception of \n{str(e_file)}')
            continue

        if not download_flag:
            logging.warning(f'Download from all mirrors failed. Please retry.')
            return

        logger.info(f'Model {model_name} was downloaded to a tmp file.')
        logger.info(f'Copying tmp file to {model_path}.')
        with open(model_path, 'wb') as cache_file:
            shutil.copyfileobj(temp_file, cache_file)
        logger.info(f'Copied tmp model file to {model_path}.')
        temp_file.close()




class InputFeatures(object):
    """A single set of features of data."""

    def __init__(self, input_ids, input_mask, segment_ids, label_id):
        self.input_ids = input_ids
        self.input_mask = input_mask
        self.segment_ids = segment_ids
        self.label_id = label_id


def _truncate_seq_pair(tokens_a, tokens_b, max_length):
    """Truncates a sequence pair in place to the maximum length."""

    # This is a simple heuristic which will always truncate the longer sequence
    # one token at a time. This makes more sense than truncating an equal percent
    # of tokens from each, since if one sequence is very short then each token
    # that's truncated likely contains more information than a longer sequence.
    while True:
        total_length = len(tokens_a) + len(tokens_b)
        if total_length <= max_length:
            break
        if len(tokens_a) > len(tokens_b):
            tokens_a.pop()
        else:
            tokens_b.pop()


def convert_example_to_feature(example_row):
    # return example_row
    example, label_map, max_seq_length, tokenizer, output_mode = example_row

    tokens_a = tokenizer.tokenize(example.text_a)

    tokens_b = None
    if example.text_b:
        tokens_b = tokenizer.tokenize(example.text_b)
        # Modifies `tokens_a` and `tokens_b` in place so that the total
        # length is less than the specified length.
        # Account for [CLS], [SEP], [SEP] with "- 3"
        _truncate_seq_pair(tokens_a, tokens_b, max_seq_length - 3)
    else:
        # Account for [CLS] and [SEP] with "- 2"
        if len(tokens_a) > max_seq_length - 2:
            tokens_a = tokens_a[:(max_seq_length - 2)]

    tokens = ["[CLS]"] + tokens_a + ["[SEP]"]
    segment_ids = [0] * len(tokens)

    if tokens_b:
        tokens += tokens_b + ["[SEP]"]
        segment_ids += [1] * (len(tokens_b) + 1)

    input_ids = tokenizer.convert_tokens_to_ids(tokens)

    # The mask has 1 for real tokens and 0 for padding tokens. Only real
    # tokens are attended to.
    input_mask = [1] * len(input_ids)

    # Zero-pad up to the sequence length.
    padding = [0] * (max_seq_length - len(input_ids))
    input_ids += padding
    input_mask += padding
    segment_ids += padding

    assert len(input_ids) == max_seq_length
    assert len(input_mask) == max_seq_length
    assert len(segment_ids) == max_seq_length

    if output_mode == "classification":
        label_id = label_map[example.label]
    elif output_mode == "regression":
        label_id = float(example.label)
    else:
        raise KeyError(output_mode)

    return InputFeatures(input_ids=input_ids,
                         input_mask=input_mask,
                         segment_ids=segment_ids,
                         label_id=label_id)




import setuptools
with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
    name="agreementr",
    version="1.3",
    author="Junjie Wu",
    author_email="wujj38@mail2.sysu.edu.cn",
    description="agreementr",
    python_requires=">=3.6",
    include_package_data=True,
    packages=setuptools.find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    #package_data = {'':['data']}
)